import os
import re
from typing import Optional

from common.log import LogData
from .base import Parser


class LatencyLogParser(Parser):
    @staticmethod
    def extract_id(fn: str) -> Optional[str]:
        match = re.match(r"(client-\d+)\.latency", fn)
        if match is None:
            return None
        return match.group(1)

    def can_handle(self, fn: str) -> bool:
        return self.extract_id(fn) is not None

    def parse_file(self, path: str) -> LogData:
        ld = LogData("latency", self.extract_id(os.path.basename(path)), path)

        with open(path, "r") as f:
            timestamp = None
            # keep last timestamp when encountering lines without
            for line in f:
                line = line.strip()
                if not line:
                    continue

                split_line = line.split()
                if len(split_line) != 4:
                    ld.add_warning(timestamp, line)
                    continue

                try:
                    timestamp = float(int(split_line[1])) * 1e-6
                    ld.add_data(timestamp, client=int(split_line[0]), latency=int(split_line[2]), tag=split_line[3])
                except ValueError:
                    ld.add_warning(timestamp, line)

        return ld
