import os
import re
from typing import Optional

from common.log import LogData
from .base import Parser


class ResourcesLogParser(Parser):
    @staticmethod
    def extract_id(fn: str) -> Optional[str]:
        match = re.match(r"((client|server)-\d+)-resources\.log", fn)
        if match is None:
            return None
        return match.group(1)

    def can_handle(self, fn: str) -> bool:
        return self.extract_id(fn) is not None

    def parse_file(self, path: str) -> Optional[LogData]:
        ld = LogData("resources", self.extract_id(os.path.basename(path)), path)

        with open(path, "r") as f:
            timestamp = None
            # keep last timestamp when encountering lines without
            for line in f:
                line = line.strip()
                if not line:
                    continue

                parts = line.split(None, 2)
                try:
                    timestamp = float(parts[0])
                    if len(parts) == 1:
                        # should only happen from time to time for the last line in the logfile
                        continue
                    cpu_percent = float(parts[1])
                    memory_percent = float(parts[2])
                    ld.add_data(timestamp, cpu_percent=cpu_percent, memory_percent=memory_percent)
                except (IndexError, ValueError):
                    ld.add_warning(timestamp, line)

        return ld
