import os
import re
from typing import Optional

from common.log import LogData
from .base import Parser


class ServerLogParser(Parser):
    @staticmethod
    def extract_id(fn: str) -> Optional[str]:
        match = re.match(r"(server-\d+)\.log", fn)
        if match is None:
            return None
        return match.group(1)

    def can_handle(self, fn: str) -> bool:
        return self.extract_id(fn) is not None

    def parse_file(self, path: str) -> Optional[LogData]:
        ld = LogData("server", self.extract_id(os.path.basename(path)), path)

        with open(path, "r") as f:
            at_start = True
            timestamp = None
            for line in f:
                line = line.strip()
                if not line:
                    continue

                split_line = line.split(None, 1)
                if split_line[0].isdigit():
                    timestamp = float(int(split_line[0])) * 1e-6
                    # remove timestamp at line start
                    line = split_line[1]
                # keep last timestamp when encountering lines without

                if at_start:
                    if line == "main [EVENT] RPLCA: READY":
                        at_start = False
                        ld.add_data(timestamp, started=1)
                    continue

                ld.add_warning(timestamp, line)

        return ld
