#!/usr/bin/env python3

import os
from collections import defaultdict
from typing import Optional


def parse_ping_log(fn: str):
    data = {}

    who = None
    with open(fn, "r") as f:
        started = False
        for line in f:
            parts = line.split()
            if parts[0] == "[BENCH]":
                parts = parts[1:]

            if parts[0] == "===":
                started = True
                who = (parts[3], parts[5])
            elif parts[0] == "rtt":
                if who is None:
                    raise Exception("Broken logfile " + fn)

                times = parts[3].split("/")
                data[who] = float(times[1])
                if float(times[3]) > 5:
                    print("High variance between {}".format(who))

                who = None
            elif started:
                print("Unknown line " + line)

    return data


def strip_folder_name(name: str) -> Optional[str]:
    name_chunks = name.split("-measure_server_pings-", 1)
    if len(name_chunks) < 2:
        return None

    scenario = name_chunks[1].rsplit("-", 2)[0]
    return scenario


def analyze(folder_name: str):
    dir_map = defaultdict(list)
    for root, dirs, files in os.walk(folder_name):
        for dir_name in dirs:
            scenario = strip_folder_name(dir_name)
            if scenario is None:
                print("Skipping folder {}".format(dir_name))
                continue
            dir_map[scenario].append(os.path.join(root, dir_name))
        break

    for dir_list in dir_map.values():
        dir_list.sort()

    scenarios = sorted([k for k in dir_map.keys()])

    for scenario in scenarios:
        dir_list = dir_map[scenario]
        process_pings(dir_list, scenario)


def process_pings(dir_list, scenario):
    for dir_name in dir_list:
        data = parse_ping_log(os.path.join(dir_name, "test.log"))
        print(scenario)
        for (who, latency) in data.items():
            print("PING_TIME_MAP['{}']['{}'] = {}".format(who[0], who[1], latency))


def main():
    analyze(".")


if __name__ == '__main__':
    main()
