#!/usr/bin/env python3
import argparse
import subprocess
from typing import Sequence

from benchmark import GenericTestRun, main


class MeasureServerPings(GenericTestRun):
    DISTRIBUTE_TO_TYPES = []  # type: Sequence[str]

    def __init__(self, *args):
        super().__init__(*args)
        self.duration = self.args.duration

    @classmethod
    def setup_args(cls, parser: argparse.ArgumentParser) -> None:
        parser.add_argument("duration", help="Duration of the test run", type=int)

    def stage_bench(self) -> bool:
        servers = self.get_servers(("client.network.addresses", "replica.network.addresses"))

        for (i, dst) in enumerate(servers):
            for j in range(i + 1, len(servers)):
                src = servers[j]
                self.log("=== Ping from {} to {} ===".format(src, dst))
                ssh = self.start_ssh(self.config["remote.user"], src, "ping", "-c", str(self.duration), dst,
                                     stdout=subprocess.PIPE)
                (output, _) = ssh.communicate()
                if ssh.returncode != 0:
                    self.log("Measurement error {}".format(ssh.returncode))
                    return False

                for line in output.decode("utf-8").splitlines():
                    if "rtt min/avg/max/mdev" in line:
                        self.log(line)

        return True


if __name__ == '__main__':
    main(MeasureServerPings)
