package refit.agreement.weave;

import java.nio.ByteBuffer;
import java.util.Objects;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITOrderRequest;
import refit.replica.order.REFITOrderProtocolMessage;


public class REFITWeaveMessages {

	public static class REFITWeaveMessagePrepare extends REFITOrderProtocolMessage {

		private static final int PREPARE_HEADER_SIZE = Integer.BYTES;

		public final REFITOrderRequest request;

		public REFITWeaveMessagePrepare(long instanceID, int viewID, REFITOrderRequest request, short from) {
			super(REFITMessageType.WEAVE_PREPARE, instanceID, viewID, from);
			this.request = request;
		}

		public REFITWeaveMessagePrepare(ByteBuffer buffer) {
			super(REFITMessageType.WEAVE_PREPARE, buffer);
			int requestSize = buffer.getInt();
			ByteBuffer requestBuffer = buffer.slice();
			requestBuffer.limit(requestSize);
			request = (REFITOrderRequest) REFITMessage.createMessage(requestBuffer);
			buffer.position(buffer.position() + requestSize);
		}

		@Override
		public REFITMessageType groupType() {
			return REFITMessageType.ORDER;
		}

		@Override
		public AuthenticationType getAuthenticationType() {
			return AuthenticationType.NONE;
		}

		@Override
		protected void serialize(final ByteBuffer buffer) {
			super.serialize(buffer);
			int requestSize = request.calculateMessageSize();
			buffer.putInt(requestSize);
			ByteBuffer requestBuffer = buffer.slice();
			requestBuffer.limit(requestSize);
			request.serializeInto(requestBuffer);
			buffer.position(buffer.position() + requestSize);
		}

		@Override
		protected int getHeaderSize() {
			return super.getHeaderSize() + PREPARE_HEADER_SIZE;
		}

		@Override
		protected int calculatePayloadSize() {
			return super.calculatePayloadSize() + PREPARE_HEADER_SIZE + request.calculateMessageSize();
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;
			if (!super.equals(o)) return false;
			REFITWeaveMessagePrepare that = (REFITWeaveMessagePrepare) o;
			return Objects.equals(request, that.request) && Objects.equals(viewID, that.viewID);
		}

		@Override
		public int hashCode() {
			return Objects.hash(super.hashCode(), request, viewID);
		}
	}

	public static class REFITWeaveMessageCommit extends REFITOrderProtocolMessage {

		public REFITWeaveMessageCommit(long instanceID, int viewID, short from) {
			super(REFITMessageType.WEAVE_COMMIT, instanceID, viewID, from);
		}

		public REFITWeaveMessageCommit(ByteBuffer buffer) {
			super(REFITMessageType.WEAVE_COMMIT, buffer);
		}

		@Override
		public REFITMessageType groupType() {
			return REFITMessageType.ORDER;
		}

		@Override
		public REFITMessage.AuthenticationType getAuthenticationType() {
			return REFITMessage.AuthenticationType.NONE;
		}
	}
}
