package refit.application;

import java.nio.ByteBuffer;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import refit.message.REFITUniqueID;
import refit.replica.checkpoint.REFITCheckpointObject;
import refit.replica.fuzzycheckpoint.REFITFuzzyCheckpointContainer;

public class REFITNopSystemServer implements REFITSystemApplicationServer {

	public REFITNopSystemServer() {
	}

	@Override
	public void init() {
		// nothing to do as no state exists
	}

	@Override
	public boolean checkReadOnly(ByteBuffer payload) {
		return false;
	}

	@Override
	public REFITResult processRequest(REFITUniqueID uid, long agreementSeqNr, ByteBuffer payload, boolean createUpdate) {
		return new REFITResult(ByteBuffer.wrap(new byte[0]), (createUpdate) ? ByteBuffer.wrap(new byte[0]) : null);
	}

	@Override
	public void applyUpdate(ByteBuffer update) {
		// nothing to do as no state exists
	}

	@Override
	public REFITLeaderRecommendation queryLeader(int lastLeader, long targetProbeCounter) {
		// No recommendation
		return null;
	}

	@Override
	public REFITCheckpointObject[] createCheckpoint() {
		REFITCheckpointObject[] objects = new REFITCheckpointObject[1];
		objects[0] = new REFITCheckpointObject(new byte[0]);
		return objects;
	}

	@Override
	public void applyCheckpoint(REFITCheckpointObject[] checkpoint) {
		// nothing to do as no state exists
	}

	@Override
	public REFITFuzzyCheckpointContainer<? extends REFITCheckpointObject, ? extends REFITCheckpointObject> createFuzzyCheckpoint() {
		throw new InternalError("not implemented yet");
	}

	@Override
	public Set<?> takeChangelist() {
		throw new InternalError("not implemented yet");
	}

	@Override
	public REFITCheckpointObject getObject(Object key) {
		throw new InternalError("not implemented yet");
	}

	@Override
	public Map<Object, REFITCheckpointObject> createDiffCheckpoint(Set<?> changelist) {
		throw new InternalError("not implemented yet");
	}

	@Override
	public Map<Object, REFITCheckpointObject> createFuzzyDiffCheckpoint(Set<?> changelist) {
		throw new InternalError("not implemented yet");
	}

	@Override
	public Collection<REFITCheckpointObject> createFuzzyUpdateCheckpoint(Map<?, REFITCheckpointObject> changelist) {
		throw new InternalError("not implemented yet");
	}
}
