package refit.application.counter;

import java.nio.ByteBuffer;
import java.util.Random;

import refit.client.REFITClientLibrary;
import refit.client.REFITClientLibrary.RequestMode;
import refit.client.REFITClientTask;
import refit.config.REFITConfig;
import refit.util.REFITAssert;
import refit.util.REFITTime;

public class REFITCounterClientTask extends REFITClientTask {

	private final ByteBuffer request;
	private final ByteBuffer requestReadOnly;

	private int invocationCounter;
	private long invocationTime;
	private boolean alive;
	private boolean readOnly;
	private final Random rand;
	private long lastRequestSeqNr;


	public REFITCounterClientTask(REFITClientLibrary service) {
		super(service);
		this.request = ByteBuffer.wrap(new byte[REFITConfig.REQUEST_SIZE]);
		this.requestReadOnly = ByteBuffer.wrap(new byte[(REFITConfig.REQUEST_SIZE > 0) ? REFITConfig.REQUEST_SIZE : 1]);
		this.requestReadOnly.put((byte) 0xff);
		this.requestReadOnly.position(0);

		this.invocationCounter = 0;
		this.invocationTime = -1L;
		this.alive = true;

		rand = new Random(service.getNodeID());
		this.lastRequestSeqNr = 0;
		REFITAssert.assertTrue(!REFITConfig.RANDOM_DELAY_AFTER_REQUEST, "RANDOM_DELAY_AFTER_REQUEST not implemented for client task");
	}


	@Override
	public String toString() {
		return "COCLNT[" + service.getNodeID() + "]";
	}


	// ##################
	// # SCHEDULER TASK #
	// ##################

	@Override
	protected void work() {
		// Check if client is runnable
		if (!alive) return;

		// Receive stable result
		if (invocationTime > 0) {
			long now = REFITTime.nanoTime.getAsLong();
			service.getStatistics().event((long) ((now - invocationTime) / 1000f), service.getNodeID(), (readOnly) ? "r" : "w");
			if (!readOnly) lastRequestSeqNr = service.getAssignedSeqNr();
			event();

			if (readOnly || REFITConfig.WAIT_FOR_FULL_REPLY) {
				// Check result
				ByteBuffer resultBuffer = service.getResult();
				int value = resultBuffer.getInt();
				if (value != invocationCounter) {
					System.err.println("Bad result: counter value is " + value + " instead of " + invocationCounter);
					alive = false;
					return;
				}
			}
		}

		// Issue next request
		invocationTime = REFITTime.nanoTime.getAsLong();
		readOnly = (rand.nextFloat() < REFITConfig.READ_WRITE_RATIO);
		service.invokeAsync((readOnly) ? requestReadOnly.slice() : request.slice(), lastRequestSeqNr,
				(readOnly) ? RequestMode.READ_WEAK : ((REFITConfig.WAIT_FOR_FULL_REPLY) ? RequestMode.WRITE : RequestMode.WRITE_ACK));
		if (!readOnly) invocationCounter++;
	}

}
