package refit.application.rezk;

import java.nio.ByteBuffer;
import java.util.HashMap;

import refit.application.rezk.common.REZKUpdate;
import refit.replica.checkpoint.REFITCheckpointObject;
import refit.replica.fuzzycheckpoint.REFITFuzzyCheckpointContainer;

public class REZKFuzzyCheckpointContainer extends REFITFuzzyCheckpointContainer<REZKUpdate, REZKUpdate> {

	public REZKFuzzyCheckpointContainer() {
		super();
	}

	public REZKFuzzyCheckpointContainer(REFITCheckpointObject[] serializedData) {
		super(serializedData);
	}

	@Override
	public void mergeUpdate(ByteBuffer update) {
		while (update.hasRemaining()) {
			REZKUpdate data = new REZKUpdate(update.slice());
			Object key = getInputKey(data);
			switch (data.type) {
			case CREATE:
				handleCreate(key, data);
				break;
			case DELETE:
				handleDelete(key);
				break;
			case SET_DATA:
				handleSetData(key, data);
				break;
			case SET_STAT:
				handleSetStat(key, data);
				break;
			case SET_SEQ:
				handleSetSeq(key, data);
				break;
			default:
				throw new IllegalStateException("Unhandled operation: " + data.type);
			}
			update.position(update.position() + data.getMessageSize());
		}
	}

	private void handleCreate(Object key, REZKUpdate data) {
		dataMap.put(key, data);
	}

	private void handleDelete(Object key) {
		dataMap.remove(key);
	}

	private void handleSetData(Object key, REZKUpdate data) {
		REZKUpdate newUpdate = new REZKUpdate(REZKUpdate.REZKUpdateType.CREATE, data.path);
		newUpdate.setDataFrom(data);
		newUpdate.setStat(data.stat);
		newUpdate.serializeMessage();
		dataMap.put(key, newUpdate);
	}

	private void handleSetStat(Object key, REZKUpdate data) {
		REZKUpdate stored = dataMap.get(key);
		REZKUpdate newUpdate = new REZKUpdate(REZKUpdate.REZKUpdateType.CREATE, data.path);
		newUpdate.setDataFrom(stored);
		newUpdate.setStat(data.stat);
		newUpdate.serializeMessage();
		dataMap.put(key, newUpdate);
	}

	private void handleSetSeq(Object key, REZKUpdate data) {
		dataMap.put(key, data);
	}

	@Override
	protected REZKUpdate deserializeData(ByteBuffer data) {
		return new REZKUpdate(data);
	}

	@Override
	protected Object getInputKey(REZKUpdate input) {
		return input.path;
	}

	@Override
	protected Object getStorageKey(REZKUpdate storage) {
		return storage.path;
	}

	@Override
	public void mergeUpdateMarker(HashMap<Object, REFITCheckpointObject> updateChangeList, ByteBuffer payload) {
		REZKUpdate data = deserializeData(payload.slice());
		REFITCheckpointObject lastChange = updateChangeList.get(data.path);
		if (data.type != REZKUpdate.REZKUpdateType.STAT_STUB || lastChange == null) {
			updateChangeList.put(data.path, data);
		}
	}
}
