package refit.application.rezk;

import refit.client.REFITClientLibrary;
import refit.config.REFITConfig;
import refit.util.REFITTime;

public class REZKTestClientQueue extends REZKTestClient {
	private String path;
	private boolean inQueueStep = true;
	private boolean isInitialized = false;
	private int clientID;

	public REZKTestClientQueue(REFITClientLibrary service) {
		super(service);
		clientID = service.getNodeID();
		path = "/queue";
	}

	@Override
	protected void work() throws Exception {
		if (!isInitialized) {
			clearNodes();
			isInitialized = true;
		}
		if (inQueueStep) {
			queue();
		} else {
			dequeue();
		}
		inQueueStep = !inQueueStep;
	}

	private void clearNodes() throws Exception {
		for (int i = clientID; i < REFITConfig.NR_OF_NODES; i += REFITConfig.TOTAL_NR_OF_CLIENTS) {
			String np = String.format("%s%05d", "/node", i);
			client.delete(np);
		}
	}

	private void queue() throws Exception {
		byte[] data = REZKTestServer.createNodeData(clientID);
		// Invoke operation
		long startTime = REFITTime.nanoTime.getAsLong();
		client.create(path + "/element", data, true);
		long endTime = REFITTime.nanoTime.getAsLong();
		service.getStatistics().event((long) ((endTime - startTime) / 1000f), service.getNodeID(), "q");
	}

	private void dequeue() throws Exception {
		// Invoke operation
		long startTime = REFITTime.nanoTime.getAsLong();
		client.dequeue(path, null);
		long endTime = REFITTime.nanoTime.getAsLong();
		service.getStatistics().event((long) ((endTime - startTime) / 1000f), service.getNodeID(), "d");
	}
}
