package refit.application.rezk;

import refit.application.rezk.common.REZKException;
import refit.application.rezk.datatree.REZKDataTree;
import refit.application.rezk.datatree.REZKInMemoryDataTree;
import refit.client.REFITLocalClientLibrary;
import refit.config.REFITConfig;
import refit.scheduler.REFITSchedulerGroup;
import refit.util.REFITIntervalStatistics;

public class REZKTestServer extends REZKServer {

	protected REZKTestServer(REZKDataTree tree) {
		super(tree);
	}

	public REZKTestServer() {
		this(new REZKInMemoryDataTree());
	}

	@Override
	public void init() {
		super.init();

		REFITLocalClientLibrary connection = new REFITLocalClientLibrary(this, null,
				new REFITSchedulerGroup(false),
				new REFITIntervalStatistics(1000, null, null),
				null, true, null);
		REZKClient service = new REZKClient(connection);

		final int nrOfNodes = REFITConfig.NR_OF_NODES;
		for (int i = 0; i < nrOfNodes; i++) {
			String path = String.format("%s%05d", "/node", i);
			byte[] data = createNodeData(i);
			try {
				service.create(path, data, false);
			} catch (REZKException e) {
				e.printStackTrace();
				throw new InternalError("Failed to initialize");
			}
		}

		try {
			service.create("/queue", null, false);
			service.create("/counter", new byte[Integer.BYTES], false);
		} catch (REZKException e) {
			e.printStackTrace();
			throw new InternalError("Failed to initialize");
		}
	}

	public static byte[] createNodeData(int nodeID) {
		byte[] data = new byte[Math.max(2, REFITConfig.REPLY_SIZE)];
		data[0] = (byte) (nodeID & 0xff);
		data[1] = (byte) (nodeID >> 8);
		return data;
	}

	public static boolean verifyNodeData(int nodeID, byte[] data) {
		if (data.length != Math.max(2, REFITConfig.REPLY_SIZE)) return false;
		if (data[0] != (byte) (nodeID & 0xff) || data[1] != (byte) (nodeID >> 8)) return false;
		for (int i = 2; i < data.length; i++) {
			if (data[i] != 0) {
				return false;
			}
		}
		return true;
	}
}
