package refit.application.rezk.util;

import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

public class REFITByteString implements Comparable<REFITByteString> {
	public static final Charset UTF8 = StandardCharsets.UTF_8;

	private static final byte[] EMPTY = new byte[0];

	private final byte[] data;
	private int hash = -1;

	public REFITByteString(String string) {
		this(string.getBytes(UTF8));
	}

	public REFITByteString() {
		this(EMPTY);
	}

	public REFITByteString(byte[] data) {
		this.data = data;
	}

	public int length() {
		return data.length;
	}

	public byte[] getBytes() {
		return data;
	}

	@Override
	public int hashCode() {
		if (hash == -1) {
			hash = Arrays.hashCode(data);
			// make sure that the hash calculation is only done once!
			if (hash == -1) {
				hash = -2;
			}
		}
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null || getClass() != obj.getClass()) return false;
		REFITByteString other = (REFITByteString) obj;
		if (this.hash != -1 && other.hash != -1 && this.hash != other.hash) return false;
		return Arrays.equals(data, other.data);
	}

	@Override
	public String toString() {
		return new String(data, UTF8);
	}

	public int lastIndexOf(byte c) {
		for (int i = data.length - 1; i >= 0; i--) {
			if (data[i] == c) {
				return i;
			}
		}
		return -1;
	}

	public REFITByteString substring(int beginIndex, int endIndex) {
		if (beginIndex < 0) {
			throw new StringIndexOutOfBoundsException(String.format("Begin: %d, End: %d", beginIndex, endIndex));
		} else if (endIndex > length()) {
			throw new StringIndexOutOfBoundsException(String.format("Begin: %d, End: %d", beginIndex, endIndex));
		}
		int substrLength = endIndex - beginIndex;
		if (substrLength < 0) {
			throw new StringIndexOutOfBoundsException(String.format("Begin: %d, End: %d", beginIndex, endIndex));
		}
		return new REFITByteString(Arrays.copyOfRange(data, beginIndex, endIndex));
	}

	public REFITByteString substring(int beginIndex) {
		return substring(beginIndex, length());
	}

	public REFITByteString concat(REFITByteString str) {
		if (str.length() == 0) {
			return this;
		}
		byte[] merged = Arrays.copyOf(data, this.length() + str.length());
		System.arraycopy(str.data, 0, merged, this.length(), str.length());
		return new REFITByteString(merged);
	}

	@Override
	public int compareTo(REFITByteString o) {
		return Arrays.compare(data, o.data);
	}
}
