package refit.message;

import java.nio.ByteBuffer;

import refit.config.REFITConfig;

public class REFITProbeData extends REFITBaseMessage {

	private static final int HEADER_SIZE = Short.BYTES + Short.BYTES * REFITConfig.TOTAL_NR_OF_REPLICAS + 2 * Short.BYTES * REFITConfig.TOTAL_NR_OF_REPLICAS * REFITConfig.TOTAL_NR_OF_REPLICAS;

	public final short primaryID;
	public final short[] replyTimes;
	public final short[][] preparedTimes;
	public final short[][] committedTimes;


	public REFITProbeData(short primaryID, short[] replyTimes, short[][] preparedTimes, short[][] committedTimes) {
		if (replyTimes.length != REFITConfig.TOTAL_NR_OF_REPLICAS) {
			throw new IllegalArgumentException("Wrong number of replies????");
		}
		this.primaryID = primaryID;
		this.replyTimes = replyTimes;
		this.preparedTimes = preparedTimes;
		this.committedTimes = committedTimes;
	}

	public REFITProbeData(ByteBuffer buffer) {
		super(buffer);
		this.primaryID = buffer.getShort();
		this.replyTimes = new short[REFITConfig.TOTAL_NR_OF_REPLICAS];
		for (int i = 0; i < replyTimes.length; i++) {
			replyTimes[i] = buffer.getShort();
		}
		this.preparedTimes = new short[REFITConfig.TOTAL_NR_OF_REPLICAS][];
		for (int i = 0; i < preparedTimes.length; i++) {
			preparedTimes[i] = new short[REFITConfig.TOTAL_NR_OF_REPLICAS];
			for (int j = 0; j < preparedTimes[i].length; j++) {
				preparedTimes[i][j] = buffer.getShort();
			}
		}
		this.committedTimes = new short[REFITConfig.TOTAL_NR_OF_REPLICAS][];
		for (int i = 0; i < committedTimes.length; i++) {
			committedTimes[i] = new short[REFITConfig.TOTAL_NR_OF_REPLICAS];
			for (int j = 0; j < committedTimes[i].length; j++) {
				committedTimes[i][j] = buffer.getShort();
			}
		}
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		buffer.putShort(primaryID);
		for (short replyTime : replyTimes) {
			buffer.putShort(replyTime);
		}
		for (short[] times : preparedTimes) {
			for (short time : times) {
				buffer.putShort(time);
			}
		}
		for (short[] times : committedTimes) {
			for (short time : times) {
				buffer.putShort(time);
			}
		}
	}

	@Override
	protected int getHeaderSize() {
		return (super.getHeaderSize() + HEADER_SIZE);
	}

	@Override
	protected int calculatePayloadSize() {
		return (super.calculatePayloadSize() + HEADER_SIZE);
	}

}
