package refit.message;

import java.nio.ByteBuffer;

import refit.config.REFITConfig;
import refit.util.REFITPayload;

public class REFITProbeResult extends REFITBaseMessage {

	private static final int HEADER_SIZE = Short.BYTES + 2 * Short.BYTES * REFITConfig.TOTAL_NR_OF_REPLICAS;

	public final byte[] executeRiddle;
	public final short[] preparedTimes;
	public final short[] commitTimes;

	public REFITProbeResult(byte[] executeRiddle, short[] preparedTimes, short[] commitTimes) {
		if (preparedTimes.length != REFITConfig.TOTAL_NR_OF_REPLICAS || commitTimes.length != REFITConfig.TOTAL_NR_OF_REPLICAS) {
			throw new IllegalArgumentException("Wrong number of timestamps????");
		}
		if (executeRiddle.length != REFITPayload.HASH_SIZE + (REFITConfig.TOTAL_NR_OF_REPLICAS + 1) * (Short.SIZE / 8)) {
			throw new IllegalArgumentException("Invalid merged commit riddle");
		}
		this.executeRiddle = executeRiddle;
		this.preparedTimes = preparedTimes;
		this.commitTimes = commitTimes;
	}

	public REFITProbeResult(ByteBuffer buffer) {
		super(buffer);

		preparedTimes = new short[REFITConfig.TOTAL_NR_OF_REPLICAS];
		for (int i = 0; i < preparedTimes.length; i++) {
			preparedTimes[i] = buffer.getShort();
		}
		commitTimes = new short[REFITConfig.TOTAL_NR_OF_REPLICAS];
		for (int i = 0; i < commitTimes.length; i++) {
			commitTimes[i] = buffer.getShort();
		}

		executeRiddle = new byte[buffer.getShort()];
		buffer.get(executeRiddle);
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		for (short preparedTime : preparedTimes) {
			buffer.putShort(preparedTime);
		}
		for (short commitTime : commitTimes) {
			buffer.putShort(commitTime);
		}

		buffer.putShort((short) executeRiddle.length);
		buffer.put(executeRiddle);
	}

	@Override
	protected int getHeaderSize() {
		return (super.getHeaderSize() + HEADER_SIZE);
	}

	@Override
	protected int calculatePayloadSize() {
		return (super.calculatePayloadSize() + HEADER_SIZE + executeRiddle.length);
	}

	private boolean isComplete = false;

	public boolean isComplete() {
		if (isComplete) {
			return true;
		}

		for (short time : preparedTimes) {
			if (time == REFITConfig.PROBE_TIMEOUT) {
				return false;
			}
		}
		for (short time : commitTimes) {
			if (time == REFITConfig.PROBE_TIMEOUT) {
				return false;
			}
		}

		isComplete = true;
		return true;
	}

}
