package refit.replica;

import java.util.Arrays;

import refit.message.REFITMessage;

public class REFITObserver<M extends REFITMessage> {

	private REFITStage[] listeners;

	public REFITObserver() {
		this.listeners = new REFITStage[0];
	}

	public void broadcast(M message) {
		for (REFITStage listener : listeners) {
			listener.insertMessage(message);
		}
	}

	public void addListener(REFITStage stage) {
		if (stage != null) {
			for (REFITStage registeredStage : listeners) {
				if (registeredStage == stage) {
					throw new AssertionError("A stage must only be registered once for each listener");
				}
			}
			listeners = addStage(listeners, stage);
		}
	}

	public void addListenersFrom(REFITObserver observer) {
		for (REFITStage listener : observer.listeners) {
			addListener(listener);
		}
	}

	private REFITStage[] addStage(REFITStage[] stages, REFITStage stage) {
		stages = Arrays.copyOf(stages, stages.length + 1);
		stages[stages.length - 1] = stage;
		return stages;
	}

}
