package refit.replica.mccoupling;

import java.nio.ByteBuffer;

import refit.config.REFITConfig;
import refit.message.REFITMessage;
import refit.message.REFITOrderRequest;
import refit.replica.checkpoint.REFITCheckpointObject;


public class REFITOrderCouplingState extends REFITCheckpointObject {
	public static final int INSTANCE_HISTORY = REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS;

	public final long[] nodeProgresses;
	public final REFITOrderRequest[] requests;

	public REFITOrderCouplingState(long agreementSeqNr, long[] nodeProgresses, REFITOrderRequest[] requests) {
		this.requests = requests;
		this.nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		System.arraycopy(nodeProgresses, 0, this.nodeProgresses, 0, this.nodeProgresses.length);

		if (requests.length != Math.min(agreementSeqNr, INSTANCE_HISTORY)) {
			throw new AssertionError("Wrong number of requests!");
		}
	}

	public REFITOrderCouplingState(long agreementSeqNr, ByteBuffer buffer) {
		super(buffer);

		this.nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeProgresses.length; i++) {
			nodeProgresses[i] = buffer.getLong();
		}
		this.requests = new REFITOrderRequest[(int) Math.min(agreementSeqNr, INSTANCE_HISTORY)];
		for (int i = 0; i < requests.length; i++) {
			requests[i] = (REFITOrderRequest) REFITMessage.createMessage(buffer.slice());
			buffer.position(buffer.position() + requests[i].getMessageSize());
		}

		messageSize = buffer.position();
	}

	@Override
	public String toString() {
		return "{REFITOrderCouplingState}";
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		for (long clientProgress : nodeProgresses) buffer.putLong(clientProgress);
		for (REFITOrderRequest request : requests) {
			buffer.put(request.getBuffer());
		}
	}

	@Override
	protected int calculatePayloadSize() {
		int messageSize = 0;
		for (REFITOrderRequest request : requests) {
			messageSize += request.getMessageSize();
		}

		return Long.BYTES * nodeProgresses.length + messageSize;
	}
}
