package refit.replica.multichannel;

import java.nio.ByteBuffer;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITUniqueID;

public class REFITChannelAck extends REFITMessage {

	private static final int HEADER_SIZE = Short.BYTES;

	public final short[] changedSubChannels;
	public final long[] changedAcks;

	public REFITChannelAck(REFITMessageType type, long seqNr, short from, short[] changedSubChannels, long[] changedAcks) {
		super(type, new REFITUniqueID((short) -1, seqNr), from);

		this.changedSubChannels = changedSubChannels;
		this.changedAcks = changedAcks;

		if (changedSubChannels.length != changedAcks.length) {
			throw new IllegalArgumentException("Both arrays must have the same size!");
		}
	}

	public REFITChannelAck(REFITMessageType type, ByteBuffer buffer) {
		super(type, buffer);

		short count = buffer.getShort();
		changedSubChannels = new short[count];
		changedAcks = new long[count];

		for (int i = 0; i < count; i++) {
			changedSubChannels[i] = buffer.getShort();
			changedAcks[i] = buffer.getLong();
		}
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		buffer.putShort((short) changedSubChannels.length);
		for (int i = 0; i < changedSubChannels.length; i++) {
			buffer.putShort(changedSubChannels[i]);
			buffer.putLong(changedAcks[i]);
		}
	}

	@Override
	public REFITMessageType groupType() {
		return REFITMessageType.CHANNEL_MSG;
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.UNICAST_MAC;
	}

	@Override
	public String toString() {
		return String.format("{%s|%d|%s}", type, from, uid);
	}

	@Override
	protected int getHeaderSize() {
		return super.getHeaderSize() + HEADER_SIZE;
	}

	@Override
	protected int calculatePayloadSize() {
		return super.calculatePayloadSize() + HEADER_SIZE + changedSubChannels.length * (Short.BYTES + Long.BYTES);
	}
}
