package refit.replica.order;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.*;
import refit.message.REFITInstruction.REFITExecuteRequest;
import refit.replica.REFITBaseReplica;


public abstract class REFITOrderProtocolInstance {

	protected final REFITBaseReplica replica;
	private final REFITOrderStageSlot slot;

	public long instanceID;
	public int viewID;

	public REFITOrderProtocolInstance(REFITOrderStageSlot slot) {
		this.replica = slot.replica;
		this.slot = slot;
		this.instanceID = -1L;
		this.viewID = -1;
	}


	// ##############
	// # LIFE CYCLE #
	// ##############

	private boolean complete;


	public void init(long instanceID, int viewID) {
		this.instanceID = instanceID;
		this.viewID = viewID;
		this.complete = false;
	}

	// execution may be triggered by unexpected messages from other replicas!
	public abstract void execute();

	public abstract REFITOrderProtocolData abort();

	protected void prepareReceived(short from) {
		slot.prepareReceived(from);
	}

	protected void commitReceived(short from) {
		slot.commitReceived(from);
	}

	public boolean isComplete() {
		return complete;
	}

	protected void completeWithResult(REFITOrderRequest result, Object attachment) {
		if (REFITLogger.LOG_ORDER) REFITLogger.logOrder(this, "Completed " + result);
		// Complete instance by producing a result
		complete = true;

		if (result == null) {
			throw new InternalError("Result must not be empty");
		}

		// Handle result
		if (result.type != REFITMessageType.REQUEST_NO_OP) {
			trackResult(result, instanceID);

			// Notify order stage
			if (result.type == REFITMessageType.BATCH) {
				for (REFITRequest request : ((REFITBatch) result).requests) {
					slot.messageStable(request.uid);
					trackResult(request, instanceID);
				}
			} else {
				slot.messageStable(result.uid);
			}
		}

		// Deliver stable result
		REFITExecuteRequest operation = new REFITExecuteRequest(instanceID, viewID, result, attachment);

		slot.sendRequest(operation);
	}

	protected void ackStable(REFITAckReply ack) {
		slot.ackStable(ack);
	}

	private void trackResult(REFITMessage result, long agreementSeqNr) {
		result.getContext().agreementSeqNr = agreementSeqNr;
		if (REFITConfig.COLLECT_MESSAGE_STATISTICS) result.getContext().statistics.track("DEL");
	}


	// ####################
	// # MESSAGE HANDLING #
	// ####################

	protected REFITMessage fetchMessage(REFITMessageType type, int viewID) {
		return slot.fetchMessage(type, viewID);
	}


	// #####################
	// # PROPOSAL HANDLING #
	// #####################

	public abstract boolean isProposer();

	public abstract boolean isPrepared();

	// only accepts the proposal if isProposer returns true
	public abstract void assignProposal(REFITOrderRequest message);

	public abstract boolean hasProposal();
}
