package refit.replica.probe;

import java.nio.ByteBuffer;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITUniqueID;


public abstract class REFITProbeOrderMessage extends REFITMessage {

	private static final int PROBE_HEADER_SIZE = Integer.BYTES;
	protected ByteBuffer command;


	protected REFITProbeOrderMessage(REFITMessageType probeType, REFITUniqueID uid, short from, ByteBuffer command) {
		super(probeType, uid, from);
		this.command = command;
	}

	protected REFITProbeOrderMessage(REFITMessageType probeType, ByteBuffer buffer) {
		super(probeType, buffer);
		int cmdSize = buffer.getInt();
		this.command = buffer.slice();
		this.command.limit(cmdSize);
		buffer.position(buffer.position() + command.limit());
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.MAC_AUTHENTICATOR;
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		serializeCommand(buffer);
	}

	protected void serializeCommand(final ByteBuffer buffer) {
		int cmdSize = command.remaining();
		buffer.putInt(cmdSize);
		// replace command ByteBuffer after serialization to store the data only once
		ByteBuffer newCommand = buffer.slice();
		buffer.put(command);
		newCommand.limit(cmdSize);
		command = newCommand;
	}

	public ByteBuffer getCommand() {
		return command.slice();
	}

	protected int calculateCommandSize() {
		return command.remaining();
	}


	@Override
	protected int getHeaderSize() {
		return super.getHeaderSize() + PROBE_HEADER_SIZE;
	}

	@Override
	protected int calculatePayloadSize() {
		return super.calculatePayloadSize() + PROBE_HEADER_SIZE + calculateCommandSize();
	}

	@Override
	public REFITMessageType groupType() {
		// Allow selection of all subtypes by the message type PROBE_ORDER
		return REFITMessageType.PROBE_ORDER;
	}

	@Override
	public String toString() {
		return String.format("{%s|%d|%s|%d}", type, from, uid, calculateCommandSize());
	}
}
