package refit.replica.probe;

import java.nio.ByteBuffer;
import java.security.MessageDigest;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;
import refit.util.REFITPayload;

public class REFITProbeProtocolMessages {


	public static class REFITProbePrePrepare extends REFITProbeOrderMessage {
		public final REFITRequest message;
		public final byte[] riddlePart;

		public REFITProbePrePrepare(REFITUniqueID uid, short from, REFITRequest message, byte[] riddlePart) {
			super(REFITMessageType.PROBE_PBFT_PRE_PREPARE, uid, from, null);
			this.message = message;
			this.riddlePart = riddlePart;
		}

		public REFITProbePrePrepare(ByteBuffer buffer) {
			super(REFITMessageType.PROBE_PBFT_PRE_PREPARE, buffer);
			// create buffer slice to keep the invariant that the ByteBuffer passed
			// to a message constructor must be at position zero
			ByteBuffer cmd = getCommand();
			this.message = (REFITRequest) REFITMessage.createMessage(cmd);
			this.riddlePart = new byte[REFITPayload.HASH_SIZE];
			buffer.get(riddlePart);
		}

		@Override
		protected void serialize(ByteBuffer buffer) {
			super.serialize(buffer);
			buffer.put(riddlePart);
		}

		@Override
		protected void serializeCommand(final ByteBuffer buffer) {
			int cmdSize = message.calculateMessageSize();
			buffer.putInt(cmdSize);
			// replace command ByteBuffer after serialization to store the data only once
			ByteBuffer newCommand = buffer.slice();
			message.serializeInto(buffer);
			newCommand.limit(cmdSize);
			command = newCommand;
		}

		@Override
		protected int calculateCommandSize() {
			return message.calculateMessageSize();
		}

		@Override
		protected int calculatePayloadSize() {
			return super.calculatePayloadSize() + REFITPayload.HASH_SIZE;
		}

		@Override
		public String toString() {
			return String.format("{%s|%d|%s|%s}", type, from, uid, message);
		}

		@Override
		protected byte[] calculateHash() {
			// Prepare message header
			ByteBuffer messageHeader = getInternalBuffer();
			messageHeader.limit(getHeaderSize());

			// Prepare message (must be done in advance in order to prevent the hash from
			// being computed during the calculation of the REFITPBFTPrePrepare hash)
			byte[] messageHash = message.getHash();

			// Calculate hash
			MessageDigest digest = REFITPayload.getDigest();
			digest.update(messageHeader);
			digest.update(messageHash);
			digest.update(riddlePart);
			return digest.digest();
		}
	}

	public static class REFITProbePrepare extends REFITProbeOrderMessage {

		public REFITProbePrepare(REFITProbePrePrepare prePrepare, short from, byte[] prepareRiddle) {
			super(REFITMessageType.PROBE_PBFT_PREPARE, prePrepare.uid, from, ByteBuffer.wrap(prepareRiddle));
		}

		public REFITProbePrepare(ByteBuffer buffer) {
			super(REFITMessageType.PROBE_PBFT_PREPARE, buffer);
		}

	}


	public static class REFITProbeCommit extends REFITProbeOrderMessage {

		public REFITProbeCommit(REFITProbePrePrepare prePrepare, short from, byte[] commitRiddle) {
			super(REFITMessageType.PROBE_PBFT_COMMIT, prePrepare.uid, from, ByteBuffer.wrap(commitRiddle));
		}

		public REFITProbeCommit(ByteBuffer buffer) {
			super(REFITMessageType.PROBE_PBFT_COMMIT, buffer);
		}

	}
}
