package refit.replica.stewardorder;

import refit.config.REFITConfig;

public class REFITStewardGroups {
	public static final int STEWARD_TOTAL_ORDERS = REFITConfig.TOTAL_NR_OF_REPLICAS / REFITConfig.ORDER_GROUP_SIZE * REFITConfig.ORDER_GROUP_SIZE;
	public static final boolean[][] STEWARD_ORDER_GROUP = new boolean[REFITConfig.TOTAL_NR_OF_REPLICAS / REFITConfig.ORDER_GROUP_SIZE][REFITConfig.TOTAL_NR_OF_REPLICAS];
	public static final boolean[] STEWARD_REPRESENTATIVES = new boolean[REFITConfig.TOTAL_NR_OF_REPLICAS];

	static {
		for (int i = 0; i < STEWARD_ORDER_GROUP.length; i++) {
			STEWARD_REPRESENTATIVES[i * REFITConfig.ORDER_GROUP_SIZE] = true;
			for (int j = i * REFITConfig.ORDER_GROUP_SIZE; j < (i + 1) * REFITConfig.ORDER_GROUP_SIZE; j++) {
				STEWARD_ORDER_GROUP[i][j] = true;
			}
		}
	}

	public static int stewardOrderGroupID(int replicaID) {
		if (replicaID < 0 || replicaID >= STEWARD_TOTAL_ORDERS) {
			throw new IllegalArgumentException("replicaID out of bounds!");
		}
		return replicaID / REFITConfig.ORDER_GROUP_SIZE;
	}

	public static short stewardClientEGID(int id) {
		for (short i = 0; i < REFITConfig.TOTAL_NR_OF_REPLICAS; i++) {
			if (REFITConfig.LOCATION[id] == REFITConfig.LOCATION[i]) {
				return (short) stewardOrderGroupID(i);
			}
		}
		throw new IllegalArgumentException("No execution group for client");
	}

	public static boolean stewardIsGroupLeader(int replicaID) {
		if (replicaID < 0 || replicaID >= STEWARD_TOTAL_ORDERS) {
			throw new IllegalArgumentException("replicaID out of bounds!");
		}
		return (replicaID % REFITConfig.ORDER_GROUP_SIZE) == 0;
	}

	public static int stewardOrderGroupStartId(int groupID) {
		if (groupID < 0 || groupID >= STEWARD_ORDER_GROUP.length) {
			throw new IllegalArgumentException("groupID out of bounds!");
		}
		return groupID * REFITConfig.ORDER_GROUP_SIZE;
	}

	public static boolean[] stewardOrderGroup(int replicaId) {
		return STEWARD_ORDER_GROUP[stewardOrderGroupID(replicaId)];
	}

	public static boolean[] stewardOrderGroupByID(int groupId) {
		return STEWARD_ORDER_GROUP[groupId];
	}

	public static boolean[] stewardRepresentatives() {
		return STEWARD_REPRESENTATIVES;
	}

	public static int stewardOrderGroupCount() {
		return STEWARD_ORDER_GROUP.length;
	}
}
