package refit.scheduler;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import refit.config.REFITLogger;

public class REFITSchedulerGroup {

	// #############################
	// # TABLE OF UNASSIGNED TASKS #
	// #############################

	private final Map<REFITSchedulerTaskType, Collection<REFITSchedulerTask>> unassignedTasks = new HashMap<>();
	public final boolean isSingleThreaded;

	public REFITSchedulerGroup(boolean isSingleThreaded) {
		this.isSingleThreaded = isSingleThreaded;
	}

	public void registerTask(REFITSchedulerTask task) {
		synchronized (unassignedTasks) {
			Collection<REFITSchedulerTask> entry = unassignedTasks.get(task.taskType);
			if (entry == null) {
				entry = new LinkedList<>();
				unassignedTasks.put(task.taskType, entry);
			}
			entry.add(task);
		}
	}

	public Collection<REFITSchedulerTask> takeTasks(REFITSchedulerTaskType taskType) {
		synchronized (unassignedTasks) {
			return unassignedTasks.remove(taskType);
		}
	}

	public void assertAllTaskAreAssigned() {
		if (!checkTaskAssignments()) {
			REFITLogger.logError(this, "Not all tasks have been assigned to a scheduler");
			REFITLogger.logError(this, "Unassigned tasks " + dumpUnassignedTasks());
			throw new IllegalStateException("All tasks must get assigned to a scheduler");
		}
	}

	public boolean checkTaskAssignments() {
		synchronized (unassignedTasks) {
			return unassignedTasks.isEmpty();
		}
	}

	public String dumpUnassignedTasks() {
		synchronized (unassignedTasks) {
			return "" + unassignedTasks;
		}
	}

}
