package refit.test;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Random;

import threshsig.Dealer;
import threshsig.GroupKey;
import threshsig.KeyShare;
import threshsig.SigShare;

public class ThreshMicrobenchmark {

	private static final int keysize = 1024;

	private static final int k = 3;

	private static final int l = 4;

	private static Dealer d;

	private static GroupKey gk;

	private static KeyShare[] keys;

	private static final byte[] data = new byte[1024];

	private static byte[] b;

	private static final SigShare[] sigs = new SigShare[k];


	public static void main(String[] args) throws NoSuchAlgorithmException {
		(new Random()).nextBytes(data);
		final MessageDigest md = MessageDigest.getInstance("SHA-1");
		b = md.digest(data);

		// Initialize a dealer with a keysize
		d = new Dealer(keysize);

		{
			final long start = System.currentTimeMillis();
			long elapsed;
			// Generate a set of key shares
			d.generateKeys(k, l);

			elapsed = System.currentTimeMillis() - start;
			System.out.println("\tKey Gen total (ms): " + elapsed);
		}

		// This is the group key common to all shares, which
		// is not assumed to be trusted. Treat like a Public Key
		gk = d.getGroupKey();

		// The Dealer has the shares and is assumed trusted
		// This should be destroyed, unless you want to reuse the
		// Special Primes of the group key to generate a new set of
		// shares
		keys = d.getShares();

		Arrays.fill(sigs, null);

		for (int z = 0; z < 20; z++) {
			final int RUNS = 10;
			final int[] S = {3, 0, 1, 2, 10, 7, 4, 6, 8, 14, 15};

			long start = System.currentTimeMillis(), elapsed;
			for (int i = 0; i < RUNS; i++)
				sigs[i % k] = keys[S[i % k]].sign(b);
			elapsed = System.currentTimeMillis() - start;
			System.out.println("Signing total (" + RUNS + " sigs) (ms): " + elapsed
					+ " Average: " + ((float) elapsed / RUNS));

			for (int i = 0; i < k; i++)
				sigs[i] = keys[S[i]].sign(b);

			start = System.currentTimeMillis();
			for (int i = 0; i < RUNS; i++)
				SigShare.combine(b, sigs, gk.getK(), gk.getL(), gk.getModulus(), gk.getExponent());
			elapsed = System.currentTimeMillis() - start;
			System.out.println("Combine total (" + RUNS + " sigs) (ms): "
					+ elapsed + " Average: " + ((float) elapsed / RUNS));

			start = System.currentTimeMillis();
			for (int i = 0; i < RUNS; i++)
				if (!SigShare.verify(b, sigs, k, l, gk.getModulus(), gk.getExponent()))
					System.out.println("Sig Failed to verify correctly");
			elapsed = System.currentTimeMillis() - start;
			System.out.println("Verification total (" + RUNS + " sigs) (ms): "
					+ elapsed + " Average: " + ((float) elapsed / RUNS / k));
		}
	}
}
