package refit.test.ping;

import java.nio.ByteBuffer;

import refit.client.REFITClientLibrary;
import refit.config.REFITLogger;
import refit.message.REFITMessage;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;
import refit.replica.REFITBaseReplica;
import refit.replica.REFITStage;
import refit.scheduler.REFITSchedulerTaskType;
import refit.util.REFITTime;


public class REFITPingStage extends REFITStage {

	private long counter;

	public REFITPingStage(REFITBaseReplica replica) {
		// Initialize data structures
		super(REFITSchedulerTaskType.ORDER_STAGE, replica);
	}

	@Override
	public String toString() {
		return "SPING";
	}

	@Override
	public void init() {
		super.init();
		setTimeout(1000);
	}

	@Override
	public void timeout() {
		ByteBuffer data = ByteBuffer.allocate(Long.BYTES);
		data.putLong(REFITTime.timeMicroUTC.getAsLong());
		data.flip();

		REFITRequest request = new REFITRequest(new REFITUniqueID(replica.id, counter), replica.id, (short) -1, -1, data,
				(short) 0, REFITClientLibrary.RequestMode.WRITE);
		request.serializeMessage();
		if (replica.id != 3) {
			replica.replicaUnicast(request, (short) 3);
		}
//		replica.replicaMulticast(request);
		counter++;

		if ((counter % 50) == 0) {
			setTimeout(2000);
		} else {
			setTimeout(2);
		}
	}

	protected void handleMessage(REFITMessage message) {
		switch (message.type) {
		case REQUEST:
			handleRequest((REFITRequest) message);
			break;
		default:
			REFITLogger.logError(this, "Unexpected message " + message);
		}
	}

	private void handleRequest(REFITRequest message) {
		if (message.uid.nodeID == replica.id) {
			// process answer to my ping
			ByteBuffer bb = message.getPayload();

			long sendTime = bb.getLong();
			long midDelta = bb.getLong() - sendTime;
			long pingDelta = REFITTime.timeMicroUTC.getAsLong() - sendTime;

			REFITLogger.logDebug(this, "Ping from " + message.from + " time " + pingDelta + " mid " + midDelta);
			if (midDelta < 0 || midDelta > pingDelta) {
				REFITLogger.logWarning(this, "Bad clock sync!");
			}
		} else {
			// got new ping, send pong
			ByteBuffer data = ByteBuffer.allocate(2 * Long.BYTES);
			data.put(message.getPayload());
			data.putLong(REFITTime.timeMicroUTC.getAsLong());
			data.flip();

			REFITRequest request = new REFITRequest(message.uid, replica.id, (short) -1, -1,
					data, (short) 0, REFITClientLibrary.RequestMode.WRITE);
			request.serializeMessage();
			replica.replicaUnicast(request, message.from);
		}
	}

}
