provider "aws" {
  region = "us-east-2"
}

// Ohio
provider "aws" {
  alias  = "use2"
  region = "us-east-2"
}

// Frankfurt
provider "aws" {
  alias  = "euc1"
  region = "eu-central-1"
}

// Tokyo
provider "aws" {
  alias  = "apse2"
  region = "ap-southeast-2"
}

module "rewan-us-east-2c" {
  source = "../rewan"
  providers = {
    aws = aws.use2
  }
  subnets = {
    us-east-2a = "10.0.5.0/24"
  }
  counts = 1
}

module "rewan-eu-central-1c" {
  source = "../rewan"
  providers = {
    aws = aws.euc1
  }
  subnets = {
    eu-central-1a = "10.0.15.0/24"
  }
  counts = 1
}

module "rewan-ap-southeast-2c" {
  source = "../rewan"
  providers = {
    aws = aws.apse2
  }
  subnets = {
    ap-southeast-2a = "10.0.25.0/24"
  }
  counts = 1
}

module "rewan-us-east-2" {
  source = "../rewan"
  providers = {
    aws = aws.use2
  }
  subnets = {
    us-east-2b = "10.0.2.0/24"
    us-east-2c = "10.0.3.0/24"
  }
  counts = 2
}

module "rewan-eu-central-1" {
  source = "../rewan"
  providers = {
    aws = aws.euc1
  }
  subnets = {
    eu-central-1b = "10.0.12.0/24"
    eu-central-1c = "10.0.13.0/24"
  }
  counts = 2
}

module "rewan-ap-southeast-2" {
  source = "../rewan"
  providers = {
    aws = aws.apse2
  }
  subnets = {
    ap-southeast-2b = "10.0.22.0/24"
    ap-southeast-2c = "10.0.23.0/24"
  }
  counts = 2
}

locals {
  outregs = [
    module.rewan-us-east-2c,
    module.rewan-eu-central-1c,
    module.rewan-ap-southeast-2c,
    module.rewan-us-east-2,
    module.rewan-eu-central-1,
    module.rewan-ap-southeast-2,
  ]
}

output "ip" {
  value = [for s in local.outregs : s.ip]
}

output "internal_ip" {
  value = [for s in local.outregs : s.internal_ip]
}
