#! /usr/bin/env python3
import json
import subprocess

# This script reads the output provided by terraform which contains the external
# and internal ip addresses of started server. These addresses are then converted
# into a format that is usable as server configuration in scripts/config/servers-...
# Clients or servers in a location are grouped in the terraform output and are assigned
# to the same location id

# the first `client_locations` IPs are client hosts
client_locations = 10
# wrap-around client location id after `client_loc_mod`
client_loc_mod = 6
client_start_idx = 0

ip = subprocess.run(["terraform", "output", "-json", "ip"], capture_output=True, check=True)
internal_ip = subprocess.run(["terraform", "output", "-json", "internal_ip"], capture_output=True, check=True)

# print(ip.stdout)
# print(internal_ip.stdout)

# prepend information on how to access the servers
out = []
out.append("remote.user = ubuntu")
out.append("remote.path = remote-runner")

ips = json.loads(ip.stdout)
loc_idx = 0
clidx = 0
idx = 0

for loc in ips:
    for addr in loc:
        if loc_idx < client_locations:
            out.append("client{}={}".format(clidx, addr))
            out.append("client{}loc={}".format(clidx, (loc_idx + client_start_idx) % client_loc_mod))
            clidx += 1
        else:
            out.append("server{}={}".format(idx, addr))
            out.append("server{}loc={}".format(idx, loc_idx - client_locations))
            idx += 1
    loc_idx += 1

ips = json.loads(internal_ip.stdout)
loc_idx = 0
clidx = 0
idx = 0

for loc in ips:
    for addr in loc:
        if loc_idx < client_locations:
            out.append("client{}int={}".format(clidx, addr))
            clidx += 1
        else:
            out.append("server{}int={}".format(idx, addr))
            idx += 1
    loc_idx += 1

print("\n".join(out))
