# parts adapted from https://github.com/kung-foo/multiregion-terraform/blob/master/cdn/module.tf
terraform {
  required_providers {
    aws = ">= 2.40"
  }
}


data "aws_ami" "ubuntu-bionic" {
  most_recent = true
  filter {
    name   = "name"
    values = ["ubuntu/images/hvm-ssd/ubuntu-bionic-18.04-amd64-server-*"]
  }
  owners = ["099720109477"] # Canonical
}

resource "aws_vpc" "main" {
  cidr_block           = "10.0.0.0/16"
  enable_dns_hostnames = "true"
  tags = {
    Name = "rewan"
  }
}

resource "aws_internet_gateway" "default" {
  vpc_id = aws_vpc.main.id
}

resource "aws_subnet" "public" {
  count                   = length(var.subnets)
  vpc_id                  = aws_vpc.main.id
  cidr_block              = element(values(var.subnets), count.index)
  availability_zone       = element(keys(var.subnets), count.index)
  map_public_ip_on_launch = "true"
  tags = {
    Name = "rewan"
  }
}

resource "aws_route_table" "public" {
  vpc_id = aws_vpc.main.id
  route {
    cidr_block = "0.0.0.0/0"
    gateway_id = aws_internet_gateway.default.id
  }
  route {
    ipv6_cidr_block = "::/0"
    gateway_id      = aws_internet_gateway.default.id
  }
}

resource "aws_route_table_association" "public" {
  count          = length(var.subnets)
  subnet_id      = element(aws_subnet.public.*.id, count.index)
  route_table_id = aws_route_table.public.id
}

data "aws_security_group" "default" {
  vpc_id = aws_vpc.main.id
  name   = "default"
}

resource "aws_security_group" "rewan" {
  name        = "rewan-tf"
  description = "Allow ssh and refit traffic"
  vpc_id      = aws_vpc.main.id

  ingress {
    from_port   = 22
    to_port     = 22
    protocol    = "tcp"
    cidr_blocks = ["0.0.0.0/0"]
  }

  ingress {
    from_port   = 8170
    to_port     = 8199
    protocol    = "tcp"
    cidr_blocks = ["0.0.0.0/0"]
  }

  ingress {
    from_port   = 8
    protocol    = "icmp"
    to_port     = -1
    cidr_blocks = ["0.0.0.0/0"]
  }

  ingress {
    from_port   = 3
    protocol    = "icmp"
    to_port     = -1
    cidr_blocks = ["0.0.0.0/0"]
  }

  egress {
    from_port   = 0
    to_port     = 0
    protocol    = "-1"
    cidr_blocks = ["0.0.0.0/0"]
  }
}

resource "aws_key_pair" "key" {
  key_name_prefix = "user-tf"
  public_key      = "ssh-rsa ADD-YOUR-SSH-KEY-HERE user@host"
}

resource "aws_spot_instance_request" "server" {
  count = var.spot ? var.counts : 0

  spot_price           = "0.02"
  wait_for_fulfillment = true
  spot_type            = "one-time"

  ami           = data.aws_ami.ubuntu-bionic.id
  instance_type = var.instance_type
  key_name      = aws_key_pair.key.key_name
  subnet_id = element(
    aws_subnet.public.*.id,
    min(length(var.subnets) - 1, count.index),
  )
  vpc_security_group_ids               = [data.aws_security_group.default.id, aws_security_group.rewan.id]
  user_data                            = file("${path.module}/server.user-data.yml")
  instance_initiated_shutdown_behavior = "terminate"
}


resource "aws_instance" "server" {
  count = var.spot ? 0 : var.counts

  // copy of spot instance
  ami           = data.aws_ami.ubuntu-bionic.id
  instance_type = var.instance_type
  key_name      = aws_key_pair.key.key_name
  subnet_id = element(
    aws_subnet.public.*.id,
    min(length(var.subnets) - 1, count.index),
  )
  vpc_security_group_ids               = [data.aws_security_group.default.id, aws_security_group.rewan.id]
  user_data                            = file("${path.module}/server.user-data.yml")
  instance_initiated_shutdown_behavior = "terminate"
}
