package refit.agreement.steward;

import java.nio.ByteBuffer;

import org.junit.BeforeClass;
import org.junit.Test;
import refit.agreement.steward.REFITStewardMessages.REFITStewardCommit;
import refit.agreement.steward.REFITStewardMessages.REFITStewardPrePrepare;
import refit.client.REFITClientLibrary.RequestMode;
import refit.message.MessageHelper;
import refit.message.REFITMessageAuthentication;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;
import threshsig.SigShare;
import threshsig.Verifier;

import static org.junit.Assert.*;

public class REFITStewardMessagesTest {

	private static final short REPLICA_ID = 2;

	private static REFITMessageAuthentication mac;

	@BeforeClass
	public static void before() {
		mac = new REFITMessageAuthentication(REPLICA_ID);
		mac.loadStewardKeys();
	}

	private static void assertVerifierEquals(Verifier a, Verifier b) {
		assertEquals(a.getZ(), b.getZ());
		assertEquals(a.getC(), b.getC());

		assertNotNull(a.getGroupVerifier());
		assertNotNull(a.getShareVerifier());
		assertNull(b.getGroupVerifier());
		assertNull(b.getShareVerifier());
		// FIXME: currently unset
//		assertEquals(a.getGroupVerifier(), b.getGroupVerifier());
//		assertEquals(a.getShareVerifier(), b.getShareVerifier());
	}

	private static void assertShareEquals(SigShare a, SigShare b) {
		assertEquals(a.getId(), b.getId());
		assertEquals(a.getSig(), b.getSig());
		assertVerifierEquals(a.getSigVerifier(), b.getSigVerifier());
	}

	private static void assertCommitEquals(REFITStewardCommit a, REFITStewardCommit b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertEquals(a.viewID, b.viewID);
		assertEquals(a.command, b.command);
		assertShareEquals(a.share, b.share);
	}

	private static void assertCopy(REFITStewardCommit batch) {
		REFITStewardCommit batchCopy = MessageHelper.copyMessage(batch, REFITStewardCommit::new);
		assertCommitEquals(batch, batchCopy);
	}

	@Test
	public void stewardCommitSerialization() {
		REFITStewardPrePrepare pp = new REFITStewardPrePrepare(42, 0, createRequest(27, 0, false), REPLICA_ID);
		SigShare share = mac.calcSigShare(pp.calcShortHash());
		REFITStewardCommit commit = new REFITStewardCommit(pp, REPLICA_ID, share);
		assertCopy(commit);
	}

	private static REFITRequest createRequest(long seqNr, int groupID, boolean isReadOnly) {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, seqNr), (short) 1, (short) groupID, -1,
				ByteBuffer.wrap(new byte[17]), (short) -1, isReadOnly ? RequestMode.READ : RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}
}
