package refit.agreement.weave;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import refit.agreement.weave.REFITWeaveMessages.REFITWeaveMessagePrepare;
import refit.client.REFITClientLibrary.RequestMode;
import refit.message.*;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

public class REFITPrepareTest {

	private static void assertPrepareEquals(REFITWeaveMessagePrepare a, REFITWeaveMessagePrepare b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertEquals(a.viewID, b.viewID);
		assertEquals(a.request, b.request);
		assertArrayEquals(a.getHash(), b.getHash());
	}

	private static void assertCopy(REFITWeaveMessagePrepare batch) {
		REFITWeaveMessagePrepare batchCopy = MessageHelper.copyMessage(batch, REFITWeaveMessagePrepare::new);
		assertPrepareEquals(batch, batchCopy);
	}

	@Test
	public void prepareSerialization() {
		REFITRequest request = createRequest(7, false);
		REFITWeaveMessagePrepare prepare = createPrepare(27, request);
		assertCopy(prepare);

		REFITOrderRequest noop = REFITNoOpRequest.INSTANCE;
		prepare = createPrepare(28, noop);
		assertCopy(prepare);

		List<REFITRequest> requests = new ArrayList<>();
		requests.add(createRequest(41, false));
		requests.add(createRequest(42, false));
		REFITBatch batch = new REFITBatch(43, (short) 1, requests);
		batch.serializeMessage();
		prepare = createPrepare(29, batch);
		assertCopy(prepare);
	}

	private static REFITWeaveMessagePrepare createPrepare(long seqNr, REFITOrderRequest request) {
		REFITWeaveMessagePrepare prepare = new REFITWeaveMessagePrepare(seqNr, 9, request, (short) 2);
		prepare.serializeMessage();
		return prepare;
	}

	private static REFITRequest createRequest(long seqNr, boolean isReadOnly) {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, seqNr), (short) 1, (short) 0,
				0, ByteBuffer.wrap(new byte[17]), (short) 0, isReadOnly ? RequestMode.READ_WEAK : RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}
}
