package refit.application.rezk;

import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.junit.Before;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import refit.application.REFITApplicationServer;
import refit.application.REFITResult;
import refit.application.rezk.common.REZKReply;
import refit.message.REFITUniqueID;
import refit.replica.checkpoint.REFITCheckpointObject;

import static org.junit.Assert.*;

public abstract class REZKFuzzyTest {
	protected REFITApplicationServer app;
	@Parameter
	public Object description;
	@Parameter(1)
	public List<ByteBuffer> prepRequests;
	@Parameter(2)
	public ByteBuffer request;

	@Parameters(name = "{index}: {0}")
	public static Collection<Object[]> parameters() {
		return REZKTestDataProvider.createAllREZKRequests();
	}

	protected static void verifyCheckpoints(REFITCheckpointObject[] initialCheckpoint, REFITCheckpointObject[] firstCheckpoint, REFITCheckpointObject[] secondCheckpoint) {
		assertFalse(Arrays.equals(firstCheckpoint, initialCheckpoint));
		assertArrayEquals(firstCheckpoint, secondCheckpoint);
	}

	@Before
	public abstract void setUp() throws Exception;

	protected int prepareApplication() {
		if (prepRequests == null) {
			return 0;
		}

		int requestSeqNr = 0;
		for (ByteBuffer request : prepRequests) {
			REFITResult prepResult = app.processRequest(new REFITUniqueID((short) 0, requestSeqNr++), 0, request.slice(), false);
			REZKReply prepReply = new REZKReply(prepResult.reply);
			assertNull(prepReply.message);
		}

		return requestSeqNr;
	}

	protected ByteBuffer executeRequest(int requestSeqNr, boolean createUpdate) {
		REFITResult result = app.processRequest(new REFITUniqueID((short) 0, requestSeqNr), 0, request.slice(), createUpdate);
		REZKReply reply = new REZKReply(result.reply);
		assertNull(reply.message);

		return result.update;
	}
}
