package refit.application.rezk.common;

import java.nio.ByteBuffer;

import org.junit.Test;
import refit.application.rezk.util.REFITByteString;

import static org.junit.Assert.*;


public class REZKRequestTest {
	private static void assertRequestEquals(REZKRequest a, REZKRequest b) {
		assertEquals(a.operation, b.operation);
		assertEquals(a.path, b.path);
		assertArrayEquals(a.data, b.data);
		assertEquals(a.watch, b.watch);
		assertEquals(a.isSequential, a.isSequential);
		assertEquals(a.version, b.version);
	}

	private static void assertCopy(REZKRequest request) {
		request.serializeMessage();

		ByteBuffer buffer = request.getBuffer();
		REZKRequest requestMod = new REZKRequest(buffer);
		assertFalse(buffer.hasRemaining());
		assertRequestEquals(request, requestMod);
	}

	@Test
	public void create() {
		REZKRequest request = new REZKRequest(REZKOperation.CREATE, new REFITByteString("test"));
		request.setSequential(true);
		request.setData(new byte[]{3, 2, 1, 0});
		assertCopy(request);
	}

	@Test
	public void delete() {
		REZKOperation[] operations = new REZKOperation[]{REZKOperation.DELETE, REZKOperation.INCREMENT,
				REZKOperation.DEQUEUE};
		for (REZKOperation op : operations) {
			REZKRequest request = new REZKRequest(op, new REFITByteString("test"));
			assertCopy(request);
		}
	}

	@Test
	public void exists() {
		REZKOperation[] operations = new REZKOperation[]{REZKOperation.EXISTS, REZKOperation.GET_CHILDREN,
				REZKOperation.GET_DATA};
		for (REZKOperation op : operations) {
			REZKRequest request = new REZKRequest(op, new REFITByteString("test"));
			request.setWatch(true);
			assertCopy(request);
		}
	}

	@Test
	public void setData() {
		REZKRequest request = new REZKRequest(REZKOperation.SET_DATA, new REFITByteString("test"));
		request.setVersion(27);
		request.setData(new byte[]{40, 41, 42, 43, 44});
		assertCopy(request);
	}
}
