package refit.application.rezk.common;

import java.nio.ByteBuffer;

import org.junit.Test;
import refit.application.rezk.util.REFITByteString;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static refit.application.rezk.common.REZKStatTest.assertStatEquals;


public class REZKUpdateTest {
	private static void assertUpdateEquals(REZKUpdate a, REZKUpdate b) {
		assertEquals(a.type, b.type);
		assertEquals(a.path, b.path);
		if (a.type != REZKUpdate.REZKUpdateType.DELETE) {
			assertArrayEquals(a.dataArray(), b.dataArray());
			assertStatEquals(a.stat, b.stat);
		}
	}

	private static void assertCopy(REZKUpdate update) {
		update.serializeMessage();

		ByteBuffer buffer = update.getBuffer();
		REZKUpdate updateMod = new REZKUpdate(buffer);
		assertEquals(buffer.limit(), updateMod.getMessageSize());
		assertUpdateEquals(update, updateMod);
	}

	@Test
	public void create() {
		REZKUpdate update = new REZKUpdate(REZKUpdate.REZKUpdateType.CREATE, new REFITByteString("test"));
		REZKStat stat = new REZKStat();
		stat.setDataLength(4);
		stat.setNrOfChildren(2);
		stat.setVersion(42);
		update.setStat(stat);
		update.setData(new byte[]{0, 1, 2, 3});
		assertCopy(update);
	}

	@Test
	public void delete() {
		REZKUpdate update = new REZKUpdate(REZKUpdate.REZKUpdateType.DELETE, new REFITByteString("test"));
		assertCopy(update);
	}

	@Test
	public void setType() {
		REZKUpdate update = new REZKUpdate(REZKUpdate.REZKUpdateType.CREATE, new REFITByteString("test"));
		update.setType(REZKUpdate.REZKUpdateType.DELETE);
		assertCopy(update);
	}
}
