package refit.message;

import java.nio.ByteBuffer;
import java.util.function.Function;

import refit.replica.checkpoint.REFITCheckpointObject;

import static org.junit.Assert.*;

public class MessageHelper {

	public static void assertHeaderEquals(REFITMessage a, REFITMessage b) {
		assertEquals(a.type, b.type);
		assertEquals(a.groupType(), b.groupType());
		assertEquals(a.getAuthenticationType(), b.getAuthenticationType());

		assertEquals(a.from, b.from);
		assertEquals(a.uid.nodeID, b.uid.nodeID);
		assertEquals(a.uid.seqNr, b.uid.seqNr);
	}

	public static void assertBasicsEquals(REFITMessage a, REFITMessage b) {
		assertHeaderEquals(a, b);

		assertEquals(a.calculatePayloadSize(), b.calculatePayloadSize());
		assertEquals(a.calculateMessageSize(), b.calculateMessageSize());
		assertEquals(a.getHeaderSize(), b.getHeaderSize());
		assertArrayEquals(a.calculateHash(), b.calculateHash());
	}

	public static <T extends REFITMessage> T copyMessage(T msg, Function<ByteBuffer, T> constructor) {
		msg.serializeMessage();

		ByteBuffer buffer = msg.getBuffer().asReadOnlyBuffer();
		T shareCopy = constructor.apply(buffer);
		// skip padding
		buffer.position(buffer.position() + msg.getPaddingSize());
		assertFalse(buffer.hasRemaining());

		return shareCopy;
	}

	public static void assertBasicsEquals(REFITCheckpointObject a, REFITCheckpointObject b) {
		assertEquals(a.getMessageSize(), b.getMessageSize());
		assertArrayEquals(a.getHash(), b.getHash());
	}

}
