package refit.message;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import refit.client.REFITClientLibrary.RequestMode;

import static org.junit.Assert.*;

public class REFITBatchTest {

	private static void assertBatchEquals(REFITBatch a, REFITBatch b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertArrayEquals(a.requests, b.requests);
		assertArrayEquals(a.getHash(), b.getHash());
	}

	private static void assertCopy(REFITBatch batch) {
		REFITBatch batchCopy = MessageHelper.copyMessage(batch, REFITBatch::new);
		assertBatchEquals(batch, batchCopy);
	}

	@Test
	public void requestBatchSerialization() {
		List<REFITRequest> requests = new ArrayList<>();
		requests.add(createRequest(41, -1, false));
		requests.add(createRequest(42, 0, false));

		REFITBatch batch = new REFITBatch(43, (short) 1, requests);
		batch.serializeMessage();
		assertCopy(batch);
	}

	@Test
	public void splitBatchSerialization() {
		List<REFITRequest> requests = new ArrayList<>();
		requests.add(createRequest(41, 1, false));
		requests.add(createRequest(42, 0, false));
		requests.add(createRequest(43, 0, true));

		REFITBatch batch = new REFITBatch(43, (short) 1, requests);
		batch.serializeMessage();
		assertCopy(batch);

		REFITBatch b0 = batch.toGroupOnly((short) 0);
		assertNotNull(b0.requests[0]);
		assertNotNull(b0.requests[1]);
		assertNotNull(b0.requests[2]);
		b0.serializeMessage();
		assertCopy(b0);

		REFITBatch b1 = batch.toGroupOnly((short) 1);
		assertNotNull(b1.requests[0]);
		assertNotNull(b0.requests[1]);
		assertNull(b1.requests[2]);
		b1.serializeMessage();
		assertCopy(b1);

		assertArrayEquals(batch.getHash(), b0.getHash());
		assertArrayEquals(batch.getHash(), b1.getHash());
	}


	private static REFITRequest createRequest(long seqNr, int groupID, boolean isReadOnly) {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, seqNr), (short) 1, (short) groupID,
				-1, ByteBuffer.wrap(new byte[17]), (short) 0, (isReadOnly) ? RequestMode.READ : RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}
}
