package refit.replica.checkpoint;

import java.nio.ByteBuffer;

import org.junit.Test;

import static org.junit.Assert.*;

public class REFITCheckpointStateTest {

	private static void assertCheckpointEquals(REFITCheckpointState a, REFITCheckpointState b) {
		assertEquals(a.from, b.from);
		assertEquals(a.uid.nodeID, b.uid.nodeID);
		assertEquals(a.uid.seqNr, b.uid.seqNr);
		assertArrayEquals(a.data, b.data);
	}

	private static void assertCopy(REFITCheckpointState checkpoint) {
		checkpoint.serializeMessage();

		ByteBuffer buffer = checkpoint.getBuffer();
		REFITCheckpointState checkpointCopy = new REFITCheckpointState(buffer);
		// skip padding
		buffer.position(buffer.position() + checkpoint.getPaddingSize());
		assertFalse(buffer.hasRemaining());
		assertCheckpointEquals(checkpoint, checkpointCopy);
		assertArrayEquals(checkpoint.getContentHash(), checkpointCopy.getContentHash());
	}

	@Test
	public void fullCheckpointSerialization() {
		REFITCheckpointState checkpoint = createFullCheckpoint(false);
		assertCopy(checkpoint);
	}

	public static REFITCheckpointState createFullCheckpoint(boolean lightweight) {
		byte[] payload = new byte[42];
		for (int i = 0; i < payload.length; i++) {
			payload[i] = (byte) i;
		}
		REFITCheckpointObject[] data;
		if (lightweight) {
			data = new REFITCheckpointObject[]{new REFITCheckpointObject(payload)};
		} else {
			data = new REFITCheckpointObject[]{new REFITCheckpointObject(payload), new REFITCheckpointObject(payload)};
		}
		return new REFITCheckpointState(4242, (short) 2, lightweight, data);
	}

	@Test
	public void toHash() {
		REFITCheckpointState fullCheckpoint = createFullCheckpoint(false);
		assertFalse(fullCheckpoint.isLightweight());
		REFITCheckpoint checkpoint = fullCheckpoint.toCheckpoint();
		assertFalse(checkpoint.isLightweight());
		assertEquals(fullCheckpoint.uid.nodeID, checkpoint.uid.nodeID);
		assertEquals(fullCheckpoint.uid.seqNr, checkpoint.uid.seqNr);
		assertEquals(fullCheckpoint.from, checkpoint.from);
		assertArrayEquals(fullCheckpoint.getContentHash(), checkpoint.contentHash);
	}

	@Test
	public void toLightweightHash() {
		REFITCheckpointState fullCheckpoint = createFullCheckpoint(true);
		assertTrue(fullCheckpoint.isLightweight());
		REFITCheckpoint checkpoint = fullCheckpoint.toCheckpoint();
		assertTrue(checkpoint.isLightweight());
		assertEquals(fullCheckpoint.uid.nodeID, checkpoint.uid.nodeID);
		assertEquals(fullCheckpoint.uid.seqNr, checkpoint.uid.seqNr);
		assertEquals(fullCheckpoint.from, checkpoint.from);
		assertArrayEquals(fullCheckpoint.getContentHash(), checkpoint.contentHash);
	}
}
