package refit.replica.checkpoint;

import java.nio.ByteBuffer;
import java.security.MessageDigest;

import org.junit.Test;
import refit.util.REFITPayload;

import static org.junit.Assert.*;

public class REFITCheckpointTest {

	private static void assertCheckpointEquals(REFITCheckpoint a, REFITCheckpoint b) {
		assertEquals(a.from, b.from);
		assertEquals(a.uid.nodeID, b.uid.nodeID);
		assertEquals(a.uid.seqNr, b.uid.seqNr);
		assertEquals(a.isLightweight(), b.isLightweight());
		assertArrayEquals(a.contentHash, b.contentHash);
	}

	private static void assertCopy(REFITCheckpoint checkpoint) {
		checkpoint.serializeMessage();

		ByteBuffer buffer = checkpoint.getBuffer();
		REFITCheckpoint checkpointCopy = new REFITCheckpoint(buffer);
		// skip padding
		buffer.position(buffer.position() + checkpoint.getPaddingSize());
		assertFalse(buffer.hasRemaining());
		assertCheckpointEquals(checkpoint, checkpointCopy);
	}

	@Test
	public void checkpointSerialization() {
		REFITCheckpoint checkpoint = createCheckpoint(false);
		assertCopy(checkpoint);
	}

	@Test
	public void lightweightCheckpointSerialization() {
		REFITCheckpoint checkpoint = createCheckpoint(true);
		assertCopy(checkpoint);
	}

	private REFITCheckpoint createCheckpoint(boolean isLightweight) {
		MessageDigest digest = REFITPayload.getDigest();
		digest.update((byte) 42);
		byte[] contentHash = digest.digest();

		return new REFITCheckpoint(4242, (short) 2, contentHash, isLightweight);
	}

	@Test
	public void setStateMethod() {
		REFITCheckpointState fullCheckpoint = REFITCheckpointStateTest.createFullCheckpoint(false);
		REFITCheckpoint checkpoint = fullCheckpoint.toCheckpoint();
		// verify test assumption
		assertFalse(checkpoint.hasState());

		boolean success = checkpoint.setState(fullCheckpoint);
		assertTrue(success);

		assertSame(fullCheckpoint, checkpoint.getState());
		assertTrue(checkpoint.hasState());
	}
}
