package refit.replica.execution;

import java.nio.ByteBuffer;
import java.security.MessageDigest;

import org.junit.Test;
import refit.config.REFITConfig;
import refit.message.MessageHelper;
import refit.util.REFITPayload;

import static org.junit.Assert.*;

public class REFITExecutorStateTest {

	private static void assertStateEquals(REFITExecutorState a, REFITExecutorState b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertArrayEquals(a.nodeProgresses, b.nodeProgresses);
		assertArrayEquals(a.replyHashes, b.replyHashes);
		assertArrayEquals(a.nodeSeqs, b.nodeSeqs);
		assertEquals(a.targetMinView, b.targetMinView);
		assertEquals(a.targetAfterSeq, b.targetAfterSeq);
		assertEquals(a.targetProbeCounter, b.targetProbeCounter);
	}

	private static void assertCopy(REFITExecutorState checkpoint) {
		checkpoint.serializeMessage();

		ByteBuffer buffer = checkpoint.getBuffer();
		REFITExecutorState checkpointCopy = new REFITExecutorState(buffer);
		assertFalse(buffer.hasRemaining());
		assertStateEquals(checkpoint, checkpointCopy);
	}

	@Test
	public void stateSerialization() {
		REFITExecutorState checkpoint = createState();
		assertCopy(checkpoint);
	}

	public static REFITExecutorState createState() {
		long[] nodeProgresses = createFakeNodeProgresses();
		byte[][] replyHashes = createFakeReplyHashes();
		long[] nodeSeqs = createFakeNodeSeqs();
		return new REFITExecutorState(nodeProgresses, replyHashes, nodeSeqs, 1, 2, 3);
	}

	private static long[] createFakeNodeProgresses() {
		long[] nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeProgresses.length; i++) {
			nodeProgresses[i] = i + 255;
		}
		return nodeProgresses;
	}

	private static byte[][] createFakeReplyHashes() {
		byte[][] replyHashes = new byte[REFITConfig.ADDRESSES.length][];
		for (int i = 0; i < replyHashes.length; i++) {
			MessageDigest digest = REFITPayload.getDigest();
			digest.update((byte) i);
			replyHashes[i] = digest.digest();
		}
		return replyHashes;
	}

	private static long[] createFakeNodeSeqs() {
		long[] nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeProgresses.length; i++) {
			nodeProgresses[i] = i + 123;
		}
		return nodeProgresses;
	}
}
