package refit.replica.mccoupling;

import java.nio.ByteBuffer;

import org.junit.Test;
import refit.client.REFITClientLibrary.RequestMode;
import refit.config.REFITConfig;
import refit.message.MessageHelper;
import refit.message.REFITOrderRequest;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertFalse;

public class REFITOrderCouplingStateTest {

	private static void assertStateEquals(REFITOrderCouplingState a, REFITOrderCouplingState b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertArrayEquals(a.nodeProgresses, b.nodeProgresses);
		assertArrayEquals(a.requests, b.requests);
	}

	private static void assertCopy(int seqNr, REFITOrderCouplingState checkpoint) {
		checkpoint.serializeMessage();

		ByteBuffer buffer = checkpoint.getBuffer();
		REFITOrderCouplingState checkpointCopy = new REFITOrderCouplingState(seqNr, buffer);
		assertFalse(buffer.hasRemaining());
		assertStateEquals(checkpoint, checkpointCopy);
	}

	@Test
	public void stateSerialization() {
		REFITOrderCouplingState checkpoint = createState(4242);
		assertCopy(4242, checkpoint);

		REFITOrderCouplingState checkpoint2 = createState(42);
		assertCopy(42, checkpoint2);
	}

	private REFITOrderCouplingState createState(int seqNr) {
		long[] progresses = createFakeNodeProgresses();
		REFITOrderRequest[] reqs = new REFITOrderRequest[Math.min(seqNr, REFITOrderCouplingState.INSTANCE_HISTORY)];
		for (int i = 0; i < reqs.length; i++) {
			reqs[i] = createRequest(i, 0, false);
		}
		return new REFITOrderCouplingState(seqNr, progresses, reqs);
	}

	private static long[] createFakeNodeProgresses() {
		long[] nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeProgresses.length; i++) {
			nodeProgresses[i] = i + 255;
		}
		return nodeProgresses;
	}

	private static REFITRequest createRequest(long seqNr, int groupID, boolean isReadOnly) {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, seqNr), (short) 1, (short) groupID, -1,
				ByteBuffer.wrap(new byte[17]), (short) 0, isReadOnly ? RequestMode.READ : RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}
}
