package refit.replica.multichannel;

import java.nio.ByteBuffer;

import org.junit.BeforeClass;
import org.junit.Test;
import refit.client.REFITClientLibrary.RequestMode;
import refit.message.*;

import static org.junit.Assert.*;

public class REFITShareTest {

	private static final short REPLICA_ID = 2;

	private static REFITMessageAuthentication mac;

	@BeforeClass
	public static void before() {
		mac = new REFITMessageAuthentication(REPLICA_ID);
	}

	private static void assertShareEquals(REFITShare a, REFITShare b) {
		MessageHelper.assertHeaderEquals(a, b);
		assertArrayEquals(a.getPayloadHash(), b.getPayloadHash());
	}

	private static void assertFullShareEquals(REFITShare a, REFITShare b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertArrayEquals(a.getPayloadHash(), b.getPayloadHash());
		assertEquals(a.getContent(), b.getContent());
	}

	private static void assertCopy(REFITShare share) {
		REFITShare shareCopy = MessageHelper.copyMessage(share, (bb) -> new REFITShare(REFITMessageType.CHANNEL_SHARE, bb));
		assertFullShareEquals(share, shareCopy);
	}


	@Test
	public void fullShareSerialization() {
		REFITShare checkpoint = createFullShare();
		assertCopy(checkpoint);
	}

	private static REFITShare createFullShare() {
		REFITRequest request = createFakeRequest();
		return new REFITShare(REFITMessageType.CHANNEL_SHARE, 42, (short) 1, REPLICA_ID, request);
	}

	private static REFITRequest createFakeRequest() {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, 2), (short) 3, (short) -1,
				-1, ByteBuffer.wrap(new byte[100]), (short) -1, RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}


	@Test
	public void noopShareSerialization() {
		REFITShare checkpoint = createNoopShare();
		assertCopy(checkpoint);
	}

	private static REFITShare createNoopShare() {
		return new REFITShare(REFITMessageType.CHANNEL_SHARE, 42, (short) 1, REPLICA_ID, REFITNoOpRequest.INSTANCE);
	}


	@Test
	public void toHash() {
		REFITShare fullShare = createFullShare();
		fullShare.serializeMessage();
		mac.appendSignature(fullShare);

		REFITShare hashShare = fullShare.toHashOnly();
		assertShareEquals(fullShare, hashShare);

		assertCopy(hashShare);

		// signature must be retained on the hash share
		REFITShare hashShareCopy = MessageHelper.copyMessage(hashShare, (bb) -> new REFITShare(REFITMessageType.CHANNEL_SHARE, bb));
		assertTrue(mac.verifySignature(hashShareCopy, REFITMessageAuthentication.ANY_SENDER));
	}
}
