package refit.replica.stewardorder;

import java.math.BigInteger;
import java.nio.ByteBuffer;

import org.junit.BeforeClass;
import org.junit.Test;
import refit.client.REFITClientLibrary.RequestMode;
import refit.message.MessageHelper;
import refit.message.REFITMessageAuthentication;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;
import refit.replica.stewardorder.REFITStewardOrderMessages.REFITStewardAccept;
import refit.replica.stewardorder.REFITStewardOrderMessages.REFITStewardProposal;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

public class REFITStewardOrderMessagesTest {

	private static final short REPLICA_ID = 2;

	private static REFITMessageAuthentication mac;

	@BeforeClass
	public static void before() {
		mac = new REFITMessageAuthentication(REPLICA_ID);
		mac.loadStewardKeys();
	}

	private static void assertAcceptEquals(REFITStewardAccept a, REFITStewardAccept b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertEquals(a.viewID, b.viewID);
		assertEquals(a.command, b.command);
		assertArrayEquals(a.prepareHash, b.prepareHash);
		assertEquals(a.thressig, b.thressig);
	}

	private static void assertCopy(REFITStewardAccept batch) {
		REFITStewardAccept batchCopy = MessageHelper.copyMessage(batch, REFITStewardAccept::new);
		assertAcceptEquals(batch, batchCopy);
	}

	@Test
	public void stewardCommitSerialization() {
		REFITStewardProposal pp = new REFITStewardProposal(42, 0, (short) 0, createRequest(27, 0, false), new BigInteger("41"));
		REFITStewardAccept commit = new REFITStewardAccept(pp, REPLICA_ID, new BigInteger("43"));
		assertCopy(commit);
	}

	private static REFITRequest createRequest(long seqNr, int groupID, boolean isReadOnly) {
		REFITRequest request = new REFITRequest(new REFITUniqueID((short) 1, seqNr), (short) 1, (short) groupID, -1,
				ByteBuffer.wrap(new byte[17]), (short) -1, isReadOnly ? RequestMode.READ : RequestMode.WRITE);
		request.serializeMessage();
		return request;
	}
}
