package refit.replica.update;

import java.util.ArrayList;

import org.junit.Test;
import refit.message.MessageHelper;

import static org.junit.Assert.*;
import static refit.replica.update.REFITUpdateTest.createUpdate;

public class REFITBatchUpdateTest {

	private static void assertBatchEquals(REFITBatchUpdate a, REFITBatchUpdate b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertEquals(a.agreementSeqNr(), b.agreementSeqNr());
		assertEquals(a.getMaxAgreementSeqNr(), b.getMaxAgreementSeqNr());
		assertEquals(a.updates, b.updates);
	}

	private static void assertCopy(REFITBatchUpdate batch) {
		REFITBatchUpdate batchCopy = MessageHelper.copyMessage(batch, REFITBatchUpdate::new);
		assertBatchEquals(batch, batchCopy);
	}

	@Test
	public void updateSerialization() {
		REFITBatchUpdate batchUpdate = createBatch();
		assertEquals(batchUpdate.getMaxAgreementSeqNr(), 44 + 27);

		for (REFITUpdate update : batchUpdate.updates) {
			update.serializeMessage();
		}
		batchUpdate.serializeMessage();
		assertCopy(batchUpdate);
	}

	@Test
	public void updateToHash() {
		REFITBatchUpdate update = createBatch();
		REFITBatchUpdate hashUpdate = update.toHashedUpdate();

		assertFalse(hashUpdate.isFullUpdate());
		assertEquals(update.uid.seqNr, hashUpdate.uid.seqNr);
		assertEquals(update.uid.nodeID, hashUpdate.uid.nodeID);
		assertEquals(update.from, hashUpdate.from);
		assertEquals(update.agreementSeqNr(), hashUpdate.agreementSeqNr());
		assertArrayEquals(update.getPayloadHash(), hashUpdate.getPayloadHash());

		hashUpdate.serializeMessage();
		assertCopy(hashUpdate);
	}

	private REFITBatchUpdate createBatch() {
		ArrayList<REFITUpdate> updates = new ArrayList<>();
		updates.add(createUpdate(42));
		updates.add(createUpdate(43));
		updates.add(createUpdate(44));

		return new REFITBatchUpdate(42 + 27, (short) 2, updates);
	}
}
