package refit.replica.update;

import java.nio.ByteBuffer;

import org.junit.Test;
import refit.message.MessageHelper;
import refit.message.REFITUniqueID;
import refit.util.REFITPayload;

import static org.junit.Assert.*;

public class REFITUpdateTest {

	private static void assertUpdateEquals(REFITUpdate a, REFITUpdate b) {
		MessageHelper.assertBasicsEquals(a, b);
		assertEquals(a.agreementSeqNr(), b.agreementSeqNr());
		assertEquals(a.isFullUpdate(), b.isFullUpdate());
		assertEquals(a.getPayload(), b.getPayload());
		assertEquals(a.isSystem, b.isSystem);
		assertArrayEquals(a.getPayloadHash(), b.getPayloadHash());
		assertArrayEquals(a.replyHash, b.replyHash);
	}

	private static void assertCopy(REFITUpdate batch) {
		REFITUpdate batchCopy = MessageHelper.copyMessage(batch, REFITUpdate::new);
		assertUpdateEquals(batch, batchCopy);
	}

	@Test
	public void updateSerialization() {
		REFITUpdate update = createUpdate(42);
		update.serializeMessage();
		assertTrue(update.isFullUpdate());
		assertCopy(update);
	}

	@Test
	public void updateToHash() {
		REFITUpdate update = createUpdate(42);
		REFITUpdate hashUpdate = update.toHashedUpdate();

		assertFalse(hashUpdate.isFullUpdate());
		assertEquals(update.uid.seqNr, hashUpdate.uid.seqNr);
		assertEquals(update.uid.nodeID, hashUpdate.uid.nodeID);
		assertEquals(update.from, hashUpdate.from);
		assertEquals(update.agreementSeqNr(), hashUpdate.agreementSeqNr());
		assertArrayEquals(update.getPayloadHash(), hashUpdate.getPayloadHash());

		hashUpdate.serializeMessage();
		assertCopy(hashUpdate);
	}

	static REFITUpdate createUpdate(long seqNr) {
		byte[] replyHash = new byte[REFITPayload.HASH_SIZE];
		replyHash[0] = 1;
		replyHash[1] = 2;
		replyHash[2] = (byte) (3 + seqNr);
		return new REFITUpdate(new REFITUniqueID((short) 1, seqNr), (short) 2, seqNr + 27,
				ByteBuffer.wrap(new byte[17]), replyHash, false);
	}
}
