package refit.replica.fuzzycheckpoint;

import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import refit.replica.checkpoint.REFITCheckpointObject;

public abstract class REFITFuzzyCheckpointContainer<InputType extends REFITCheckpointObject, StorageTyp extends REFITCheckpointObject> {
	protected Map<Object, StorageTyp> dataMap;

	protected REFITFuzzyCheckpointContainer() {
		this.dataMap = new TreeMap<>();
	}

	protected REFITFuzzyCheckpointContainer(REFITCheckpointObject[] objects) {
		this.dataMap = deserialize(objects);
	}

	protected abstract Object getInputKey(InputType input);

	protected abstract Object getStorageKey(StorageTyp storage);

	public Object getStorageKey(ByteBuffer update) {
		StorageTyp data = deserializeData(update.slice());
		return getStorageKey(data);
	}

	public void addData(StorageTyp data) {
		Object key = getStorageKey(data);
		dataMap.put(key, data);
	}

	public abstract void mergeUpdate(ByteBuffer update);

	public void applyDiff(Map<Object, ? extends REFITCheckpointObject> diff) {
		for (Map.Entry<Object, ? extends REFITCheckpointObject> change : diff.entrySet()) {
			replaceObject(change.getKey(), change.getValue());
		}
	}

	public void replaceObject(Object key, REFITCheckpointObject value) {
		if (value == null) {
			dataMap.remove(key);
		} else {
			//noinspection unchecked
			dataMap.put(key, ((StorageTyp) value));
		}
	}

	public REFITCheckpointObject[] serialize() {
		REFITCheckpointObject[] objects = new REFITCheckpointObject[dataMap.size()];
		int ctr = 0;

		for (StorageTyp value : dataMap.values()) {
			value.serializeMessage();
			objects[ctr] = value;
			ctr++;
		}
		return objects;
	}

	private Map<Object, StorageTyp> deserialize(REFITCheckpointObject[] data) {
		TreeMap<Object, StorageTyp> map = new TreeMap<>();
		for (REFITCheckpointObject object : data) {
			StorageTyp message = deserializeData(object.getBuffer());
			Object key = getStorageKey(message);
			map.put(key, message);
		}

		return map;
	}

	protected abstract StorageTyp deserializeData(ByteBuffer buffer);

	public abstract void mergeUpdateMarker(HashMap<Object, REFITCheckpointObject> updateChangeList, ByteBuffer payload);
}
